package cz.zcu.fav.kiv.mjakubas.saf.config;

import cz.zcu.fav.kiv.mjakubas.saf.config.configs.CommandConfig;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.NetConfig;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.io.file.FileIO;
import cz.zcu.fav.kiv.mjakubas.saf.io.properties.PropertiesIO;
import cz.zcu.fav.kiv.mjakubas.saf.io.resource.ResourcesIO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.nio.file.*;
import java.util.Properties;

/**
 * Represents a manager for all possible configs that application might use.
 */
public class ConfigManager {
    private static final Logger L = LogManager.getLogger(ConfigManager.class);

    private static final String CONFIG_INTERNAL_RESOURCE_FOLDER = "settings";
    private static final String CONFIG_EXTERNAL_RESOURCE_FOLDER = "config";

    private static final String COMMAND_CONFIG_PATH = "config.cmd.properties";
    private static final String NET_CONFIG_PATH = "config.net.properties";

    private static final ConfigManager configManager = new ConfigManager();

    private NetConfig netConfig;
    private CommandConfig commandConfig;
    private RequestConfig requestConfig;

    private void loadConfigs() throws IOException {
        exportConfigs();

        Properties commandProperties =
                PropertiesIO.getProperties(CONFIG_EXTERNAL_RESOURCE_FOLDER + "/" + COMMAND_CONFIG_PATH);
        Properties netProperties =
                PropertiesIO.getProperties(CONFIG_EXTERNAL_RESOURCE_FOLDER + "/" + NET_CONFIG_PATH);

        this.commandConfig = CommandConfig.loadCommandConfig(commandProperties);
        this.netConfig = NetConfig.loadNetConfig(netProperties);
        this.requestConfig = new RequestConfig(netConfig.address + ":" + netConfig.port, netConfig.getAuthenticator()
        );
    }

    private void exportConfigs() throws IOException {
        //Path resourcePath = ResourcesIO.getPath(CONFIG_INTERNAL_RESOURCE_FOLDER);
        //FileIO.copyDirectory(resourcePath.toFile().getAbsolutePath(), CONFIG_EXTERNAL_RESOURCE_FOLDER);
        Path p = ResourcesIO.createTempDirectory(CONFIG_INTERNAL_RESOURCE_FOLDER,
                COMMAND_CONFIG_PATH, NET_CONFIG_PATH);
        // cleanup
        File folder = p.toFile();
        File[] listOfFiles = folder.listFiles();
        assert listOfFiles != null;
        for (File listOfFile : listOfFiles) {
            Files.move(listOfFile.toPath(),
                    listOfFile.toPath().resolveSibling(listOfFile.getName().replaceAll("\\d", "")));
        }

        FileIO.copyDirectory(p.toFile().getAbsolutePath(), CONFIG_EXTERNAL_RESOURCE_FOLDER);

        L.info("Settings file successfully exported!");
    }

    // get

    /**
     * Gets configs manager.
     *
     * @return configs manager
     */
    public static ConfigManager getManager() {
        return configManager;
    }

    /**
     * Gets application {@link NetConfig}.
     *
     * @return application {@link NetConfig}
     */
    public NetConfig getNetConfig() throws IOException {
        if (netConfig == null)
            loadConfigs();

        return netConfig;
    }

    /**
     * Gets application {@link CommandConfig}.
     *
     * @return application {@link CommandConfig}
     */
    public CommandConfig getCommandConfig() throws IOException {
        if (commandConfig == null)
            loadConfigs();

        return commandConfig;
    }

    /**
     * Gets {@link RequestConfig} config.
     *
     * @return {@link RequestConfig} config
     * @throws IOException if any exception happens
     */
    public RequestConfig getRequestConfig() throws IOException {
        if (requestConfig == null) {
            loadConfigs();
        }

        return requestConfig;
    }
}
