package cz.zcu.fav.kiv.mjakubas.saf.gui.controller.entity;

import cz.zcu.fav.kiv.mjakubas.saf.config.ConfigManager;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.ASimpleEntity;
import cz.zcu.fav.kiv.mjakubas.saf.gui.alert.CustomAlerts;
import cz.zcu.fav.kiv.mjakubas.saf.gui.form.nform.AForm;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequestResult;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.exception.RequestErrorException;
import javafx.beans.property.ListProperty;
import javafx.beans.property.SimpleListProperty;
import javafx.collections.FXCollections;
import javafx.scene.layout.Pane;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.util.List;

/**
 * Fetches entities and manages entity connection between front-end and back-end.
 *
 * @param <E> serviced entity
 */
public abstract class AEntityController<E extends ASimpleEntity> {

    protected ListProperty<E> fetchedEntities = new SimpleListProperty<>(FXCollections.observableArrayList());
    private boolean isFetched = false;
    private final RequestConfig requestConfig;
    protected final AForm<E> entityForm;

    protected EntitiesController entitiesController;

    // constructor

    /**
     * Instantiates new {@link AEntityController} of certain entity.
     */
    public AEntityController(
            @NotNull AForm<E> entityForm,
            @NotNull EntitiesController entitiesController) {
        this.entityForm = entityForm;
        this.entitiesController = entitiesController;
        try {
            requestConfig = ConfigManager.getManager().getRequestConfig();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    // abstract

    /**
     * Base fetch address to get all entities.
     *
     * @return fetch address to get all entities.
     */
    protected abstract String getBaseAddress();

    protected abstract String getAdditionalAddress();

    /**
     * Entity {@link Class}.
     *
     * @return entity {@link Class}
     */
    protected abstract Class<E> getEntityClass();

    /**
     * if any additional data should be fetched this should be implemented.
     */
    protected abstract void fetchAdditionalData() throws FetchException;

    // request

    /**
     * Fetches all entities.
     *
     * @throws FetchException if any exception happens
     */
    private void fetchEntities() throws FetchException {
        try {
            EntitiesRequestResult entitiesRequests = EntitiesRequests
                    .get(requestConfig, getBaseAddress(), getEntityClass());
            E[] fetchedObjects = (E[]) entitiesRequests.getDeserializedEntities();
            fetchedEntities.addAll(fetchedObjects);

            fetchAdditionalData();


            isFetched = true;
        } catch (Exception e) {
            CustomAlerts.showErrorAlert("alert.title.error", "Connection error!");
            throw new FetchException(e);
        }
    }

    /**
     * Posts entity.
     *
     * @param entity entity as object
     */
    public void postEntity(@NotNull Object entity) {
        try {
            EntitiesRequestResult results =
                    EntitiesRequests.post(requestConfig, getBaseAddress(), new Object[]{entity})[0];
            CustomAlerts.showInfoAlert("alert.header.post-entity", "alert.header.post-entity.ok");
            EntitiesController.getEntitiesController().purgeFetchedData();
        } catch (RequestErrorException e) {
            CustomAlerts.showErrorAlert("alert.header.error.post-entity", e.getMessage());
        }
    }

    /**
     * Deletes entity by its id
     *
     * @param entityID id
     */
    public void deleteEntity(int entityID) {
        try {
            String path = getBaseAddress() + RequestConfig.ID_QUERY + "/" + entityID;
            EntitiesRequestResult results
                    = EntitiesRequests.delete(requestConfig, path);
            CustomAlerts.showInfoAlert("alert.header.delete-entity", "alert.header.delete-entity.ok");
            EntitiesController.getEntitiesController().purgeFetchedData();
        } catch (RequestErrorException e) {
            CustomAlerts.showErrorAlert("alert.header.error.delete-entity", e.getMessage());
        }
    }

    /**
     * Puts for additional content query of entity.
     *
     * @param entity entity.
     * @param index  index
     */
    public void putAdditionalEntity(@NotNull Object entity, int index) {
        try {
            ASimpleEntity e = (ASimpleEntity) entity;
            String path =
                    getBaseAddress() + RequestConfig.ID_QUERY + "/" + index + getAdditionalAddress() + RequestConfig.ADD_QUERY;
            EntitiesRequestResult results =
                    EntitiesRequests.put(requestConfig, path, entity);
            CustomAlerts.showInfoAlert("alert.header.put-entity", "alert.header.put-entity.ok");
            EntitiesController.getEntitiesController().purgeFetchedData();
        } catch (RequestErrorException e) {
            CustomAlerts.showErrorAlert("alert.header.error.put-entity", e.getMessage());
        }
    }

    /**
     * Puts entity by entity as object.
     *
     * @param entity entity as object
     */
    public void putEntity(@NotNull Object entity) {
        try {
            ASimpleEntity e = (ASimpleEntity) entity;
            String path = getBaseAddress() + RequestConfig.ID_QUERY + "/" + e.getID();
            EntitiesRequestResult results =
                    EntitiesRequests.put(requestConfig, path, entity);
            CustomAlerts.showInfoAlert("alert.header.put-entity", "alert.header.put-entity.ok");
            EntitiesController.getEntitiesController().purgeFetchedData();
        } catch (RequestErrorException e) {
            CustomAlerts.showErrorAlert("alert.header.error.put-entity", e.getMessage());
        }
    }

    /**
     * Removes additional content of an entity.
     *
     * @param entity entity
     * @param index index
     */
    public void putAdditionalDeleteEntity(@NotNull Object entity, int index) {
        try {
            ASimpleEntity e = (ASimpleEntity) entity;
            String path =
                    getBaseAddress() + RequestConfig.ID_QUERY + "/" + index + getAdditionalAddress() + RequestConfig.REMOVE_QUERY;
            EntitiesRequestResult results =
                    EntitiesRequests.put(requestConfig, path, entity);
            CustomAlerts.showInfoAlert("alert.header.put-entity", "alert.header.put-entity.ok");
            EntitiesController.getEntitiesController().purgeFetchedData();
        } catch (RequestErrorException e) {
            CustomAlerts.showErrorAlert("alert.header.error.put-entity", e.getMessage());
        }
    }

    // clear

    /**
     * Clears all fetched entities.
     */
    public void clearFetchedEntities() {
        fetchedEntities.clear();
        isFetched = false;
    }

    // get

    /**
     * Gets all fetched entities.
     *
     * @return all fetched entities
     * @throws FetchException if any exception happens
     */
    public List<E> getAllFetchedEntities() throws FetchException {
        if (!isFetched) {
            fetchEntities();
        }

        return fetchedEntities.stream().toList();
    }

    public E getFetchedEntity(int entityID) throws FetchException {
        if (!isFetched) {
            fetchEntities();
        }

        E fetchedEntity = fetchedEntities.stream().filter(e -> e.getID() == entityID).toList().get(0);
        if (fetchedEntity == null) {
            throw new FetchException("");
        }

        return fetchedEntity;
    }

    public Pane getCreateFormPane() {
        return entityForm.getPostView();
    }

    public Pane getShowFormPane(E entity) {
        return entityForm.getPutView(entity);
    }
}
