package cz.zcu.fav.kiv.mjakubas.saf.gui.view;

import javafx.beans.binding.NumberBinding;
import javafx.beans.binding.StringBinding;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a view with search field and list of searchable buttons.
 * Each button can be assigned a procedure on a click.
 *
 * @param <B> button
 */
public abstract class ASearchableButtonPane<B extends ToggleButton> {

    private static final int MIN_NODE_HEIGHT = 50; //px
    private static final int NODE_RIGHT_SIDE_OFFSET = 3; //px

    protected final VBox rootPane = new VBox();
    protected final VBox searchPane = new VBox();
    protected final VBox optionalPane = new VBox();

    protected final TextField searchField = new TextField();
    protected final List<B> searchableButtons = new ArrayList<>();

    // constructor

    /**
     * Instantiates new {@link ASearchableButtonPane}.
     */
    public ASearchableButtonPane() {
        buildView();
        updateFilter();
    }

    // abstract

    /**
     * Gets search field prompt text.
     *
     * @return search field prompt text
     */
    protected abstract StringBinding getSearchFieldPrompt();

    /**
     * Gets empty button label. Empty button is a button that is shown if filter returns no matched buttons.
     *
     * @return empty button label
     */
    protected abstract StringBinding getEmptyButtonLabel();

    /**
     * Gets optional {@link Pane} that is located as a first {@link Node}.
     *
     * @return optional {@link Pane} that is located as a first {@link Node} or null
     */
    protected abstract Pane fillOptionalPane();

    // build

    private void buildView() {
        buildOptionalPane(rootPane);
        buildSearchField(rootPane);
        buildSearchPane(rootPane);
    }

    private void buildSearchField(VBox rootPane) {
        searchField.promptTextProperty().bind(getSearchFieldPrompt());
        NumberBinding widthBinding = rootPane.widthProperty().subtract(NODE_RIGHT_SIDE_OFFSET);
        searchField.maxWidthProperty().bind(widthBinding);
        searchField.minHeightProperty().bind(new SimpleIntegerProperty(MIN_NODE_HEIGHT));

        searchField.setOnKeyReleased(x -> updateFilter());

        searchField.getStyleClass().addAll("text-field-primary");

        rootPane.getChildren().add(searchField);
    }

    private void buildSearchPane(VBox rootPane) {
        ScrollPane scrollSearchPane = new ScrollPane(searchPane);
        rootPane.getChildren().add(scrollSearchPane);

        NumberBinding widthBinding = rootPane.widthProperty().subtract(NODE_RIGHT_SIDE_OFFSET);
        scrollSearchPane.maxWidthProperty().bind(widthBinding);

        scrollSearchPane.setHbarPolicy(ScrollPane.ScrollBarPolicy.NEVER);

        scrollSearchPane.getStyleClass().add("pane-secondary");
    }

    private void buildOptionalPane(VBox rootPane) {
        Pane optPane = fillOptionalPane();
        if (optPane != null) {
            rootPane.getChildren().addAll(optPane);
        }
    }

    // update

    /**
     * Updates showed buttons to reflect current filter.
     */
    private void updateFilter() {
        List<B> filteredButtons = searchableButtons
                .stream()
                .filter(x -> x.getText().toLowerCase().contains(searchField.getText().toLowerCase()))
                .toList();
        searchPane.getChildren().clear();
        searchPane.getChildren().addAll(filteredButtons);

        if (filteredButtons.size() == 0) {
            searchPane.getChildren().add(getEmptyButton());
        }
    }

    // add

    /**
     * Adds a searchable button to the searchable view.
     *
     * @param button  searchable button
     * @param onClick onclick event
     */
    public void addSearchableButton(B button, Runnable onClick) {
        searchableButtons.add(button);

        button.setOnAction(x -> {
            setFocusButton(button);
            unFocusButtons();

            onClick.run();
        });

        NumberBinding widthBinding = rootPane.widthProperty().subtract(NODE_RIGHT_SIDE_OFFSET);
        button.prefHeightProperty().bind(new SimpleIntegerProperty(MIN_NODE_HEIGHT));
        button.prefWidthProperty().bind(widthBinding);

        updateFilter();
    }

    // clear

    /**
     * Clears all buttons of a {@link ASearchableButtonPane} and updates the view.
     */
    public void clearSearchableButtons() {
        searchableButtons.clear();
        searchPane.getChildren().clear();
    }

    // styling

    private ToggleButton focusButton;

    private void setFocusButton(ToggleButton button) {
        focusButton = button;
    }

    private void unFocusButtons() {
        for (B button : searchableButtons) {
            if (button == focusButton)
                continue;
            button.setSelected(false);
        }
    }


    // get

    private ToggleButton getEmptyButton() {
        ToggleButton emptyButton = new ToggleButton();
        emptyButton.textProperty().bind(getEmptyButtonLabel());

        NumberBinding widthBinding = rootPane.widthProperty().subtract(NODE_RIGHT_SIDE_OFFSET);
        emptyButton.prefHeightProperty().bind(new SimpleIntegerProperty(MIN_NODE_HEIGHT));
        emptyButton.prefWidthProperty().bind(widthBinding);

        emptyButton.setDisable(true);
        emptyButton.setFocusTraversable(false);

        emptyButton.getStyleClass().add("btn-custom-transparent");

        return emptyButton;
    }

    /**
     * Gets {@link ASearchableButtonPane} {@link Pane}.
     *
     * @return {@link ASearchableButtonPane} {@link Pane}
     */
    public Pane getPane() {
        return rootPane;
    }

    public List<B> getSearchableButtons() {
        return searchableButtons;
    }

    public ToggleButton getFocusButton() {
        return focusButton;
    }

    // set

    /**
     * Sets if view is visible.
     *
     * @param isVisible is visible
     */
    public void setVisible(boolean isVisible) {
        rootPane.setVisible(isVisible);
    }
}
