package cz.zcu.fav.kiv.mjakubas.saf.parser.args;

import cz.zcu.fav.kiv.mjakubas.saf.io.properties.PropertiesIO;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.Token;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.Logger;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Command argument parser against expected tokenized command. Checks if expected values equals input values.
 */
public class ArgsParser {
    private static final Logger L = (Logger) LogManager.getLogger(ArgsParser.class);

    private final Properties commandProperties;

    /**
     * Instantiates new {@link ArgsParser} with {@link Properties} class of all possible command arguments.
     *
     * @param commandProperties {@link Properties} class of all possible command arguments
     */
    public ArgsParser(Properties commandProperties) {
        this.commandProperties = commandProperties;
    }

    /**
     * Parses input command against expected command. Returns result.
     *
     * @param args           input command arguments
     * @param tokens         {@link Token} of expected command
     * @param methodArgTypes extracted argument {@link Class} types
     * @param methodArgs     extracted arguments of command handler
     * @return if expected command equals input argument commands
     */
    public boolean parseArgs(@NotNull String[] args, @NotNull List<Token> tokens,
                             @NotNull List<Class<?>> methodArgTypes, @NotNull List<Object> methodArgs) {
        if (tokens.size() != args.length) {
            return false;
        }

        List<Class<?>> resultMethodArgTypes = new ArrayList<>();
        List<Object> resultMethodArgs = new ArrayList<>();
        for (int i = 0; i < args.length; i++) {
            if (!parseArg(args[i], tokens.get(i), resultMethodArgTypes, resultMethodArgs)) {
                return false;
            }
        }
        methodArgTypes.addAll(resultMethodArgTypes);
        methodArgs.addAll(resultMethodArgs);
        L.trace(Arrays.toString(args) + " successfully parsed. Match with token: " + tokens);
        return true;
    }

    /**
     * Parses input command argument against expected command argument.
     *
     * @param arg            input argument
     * @param token          argument {@link Token}
     * @param methodArgTypes extracted argument {@link Class} type
     * @param methodArgs     extracted argument
     * @return if argument is same as expected token
     */
    public boolean parseArg(@NotNull String arg, @NotNull Token token,
                            @NotNull List<Class<?>> methodArgTypes, @NotNull List<Object> methodArgs) {
        if (token.getStringToken() != null) {
            return isValidStringValue(arg, token);
        } else {
            return parseClassValue(arg, token, methodArgTypes, methodArgs);
        }
    }

    /**
     * Checks if string value equals token value.
     *
     * @param arg   input argument
     * @param token token argument
     * @return if string value equals token value
     */
    public boolean isValidStringValue(@NotNull String arg, @NotNull Token token) {
        return token.getStringToken().equalsIgnoreCase(arg);
    }

    /**
     * Parses class value against token class value.
     *
     * @param arg            input argument
     * @param token          token value
     * @param methodArgTypes extracted argument {@link Class} type
     * @param methodArgs     extracted argument
     * @return parse result
     */
    public boolean parseClassValue(@NotNull String arg, @NotNull Token token,
                                   @NotNull List<Class<?>> methodArgTypes, @NotNull List<Object> methodArgs) {
        Class<?> tokenValue = token.getClassToken();
        if (tokenValue == FileType.class) {
            return parseFileTypeValue(arg, methodArgTypes, methodArgs);
        } else {
            if (tokenValue == Integer.class) {
                return parseIntegerValue(arg, methodArgTypes, methodArgs);
            } else {
                return parseGenericClassValue(arg, token, methodArgTypes, methodArgs);
            }
        }
    }

    /**
     * Parses {@link FileType} value against {@link Class} value.
     *
     * @param arg            input argument
     * @param methodArgTypes extracted argument {@link FileType} type
     * @param methodArgs     extracted argument
     * @return parse result
     */
    public boolean parseFileTypeValue(@NotNull String arg, @NotNull List<Class<?>> methodArgTypes,
                                      @NotNull List<Object> methodArgs) {
        try {
            FileType fileType = PropertiesIO.getFileTypeValue(commandProperties, arg);
            methodArgTypes.add(fileType.getClass());
            methodArgs.add(fileType);
            return true;
        } catch (Exception e) {
            L.error("no filetype matching argument found");
            return false;
        }
    }

    /**
     * Parses {@link Integer} value against {@link Class} value.
     *
     * @param arg            input argument
     * @param methodArgTypes extracted argument {@link Integer} type
     * @param methodArgs     extracted argument
     * @return parse result
     */
    public boolean parseIntegerValue(@NotNull String arg, @NotNull List<Class<?>> methodArgTypes,
                                     @NotNull List<Object> methodArgs) {
        if (!NumberUtils.isDigits(arg)) {
            return false;
        }
        methodArgTypes.add(Integer.class);
        methodArgs.add(Integer.parseInt(arg));
        return true;
    }

    /**
     * Parses generic {@link Class} value.
     *
     * @param arg            input argument
     * @param methodArgTypes extracted argument {@link Class} type
     * @param methodArgs     extracted argument
     * @return parse result
     */
    public boolean parseGenericClassValue(@NotNull String arg, @NotNull Token token,
                                          @NotNull List<Class<?>> methodArgTypes, @NotNull List<Object> methodArgs) {
        methodArgTypes.add(token.getClassToken());
        methodArgs.add(arg);
        return true;
    }
}
