package cz.zcu.fav.kiv.mjakubas.saf;

import cz.zcu.fav.kiv.mjakubas.saf.entity.*;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.*;
import org.jetbrains.annotations.*;
import org.junit.jupiter.api.*;

import java.net.*;
import java.nio.file.*;

/**
 * Defines a test loader for tests resource entities files in json format.
 */
public class EntitiesTestLoaderTest {

    public static final String ABSENCES_FILE_NAME = "raw/absences.json";
    public static final String BUILDINGS_FILE_NAME = "raw/buildings.json";
    public static final String CLASSES_FILE_NAME = "raw/classes.json";
    public static final String EVALUATIONS_FILE_NAME = "raw/evaluations.json";
    public static final String ROOMS_FILE_NAME = "raw/rooms.json";
    public static final String SCHEDULE_ACTIONS_FILE_NAME = "raw/scheduleactions.json";
    public static final String SCHEDULE_ACTIONS_DATES_FILE_NAME = "raw/scheduleactiondates.json";
    public static final String SCHOOL_DAYS_FILE_NAME = "raw/schooldays.json";
    public static final String SCHOOL_HOURS_FILE_NAME = "raw/schoolhours.json";
    public static final String STUDENTS_FILE_NAME = "raw/students.json";
    public static final String SUBJECTS_LISTS_FILE_NAME = "raw/subjectslists.json";
    public static final String SUBJECTS_FILE_NAME = "raw/subjects.json";
    public static final String TEACHERS_FILE_NAME = "raw/teachers.json";


    /**
     * Returns all loaded entities from defined json resource file.
     *
     * @param resourceName resource name
     * @param entityClass  entity class
     * @return all loaded entities
     * @throws Exception if any error happens
     */
    public static Object[] loadTestEntities(@NotNull String resourceName, @NotNull Class<?> entityClass) throws Exception {
        ClassLoader c = EntitiesTestLoaderTest.class.getClassLoader();
        URL url = c.getResource(resourceName);
        assert url != null;
        String serialization = Files.readString(Path.of(url.toURI()));
        JSONMapper mapper = new JSONMapper();
        return mapper.deserialize(serialization, entityClass);
    }

    public static Absence[] loadAbsences() throws Exception {
        return (Absence[]) loadTestEntities(ABSENCES_FILE_NAME, Absence.class);
    }

    public static Building[] loadBuildings() throws Exception {
        return (Building[]) loadTestEntities(BUILDINGS_FILE_NAME, Building.class);
    }

    public static SchoolClass[] loadClasses() throws Exception {
        return (SchoolClass[]) loadTestEntities(CLASSES_FILE_NAME, SchoolClass.class);
    }

    public static Evaluation[] loadEvaluations() throws Exception {
        return (Evaluation[]) loadTestEntities(EVALUATIONS_FILE_NAME, Evaluation.class);
    }

    public static Room[] loadRooms() throws Exception {
        return (Room[]) loadTestEntities(ROOMS_FILE_NAME, Room.class);
    }

    public static ScheduleAction[] loadScheduleActions() throws Exception {
        return (ScheduleAction[]) loadTestEntities(SCHEDULE_ACTIONS_FILE_NAME, ScheduleAction.class);
    }

    public static ScheduleActionDate[] loadScheduleActionDates() throws Exception {
        return (ScheduleActionDate[]) loadTestEntities(SCHEDULE_ACTIONS_DATES_FILE_NAME, ScheduleActionDate.class);
    }

    public static SchoolDay[] loadSchoolDays() throws Exception {
        return (SchoolDay[]) loadTestEntities(SCHOOL_DAYS_FILE_NAME, SchoolDay.class);
    }

    public static SchoolHour[] loadSchoolHours() throws Exception {
        return (SchoolHour[]) loadTestEntities(SCHOOL_HOURS_FILE_NAME, SchoolHour.class);
    }

    public static Student[] loadStudents() throws Exception {
        return (Student[]) loadTestEntities(STUDENTS_FILE_NAME, Student.class);
    }

    public static Subject[] loadSubjects() throws Exception {
        return (Subject[]) loadTestEntities(SUBJECTS_FILE_NAME, Subject.class);
    }

    public static SubjectList[] loadSubjectsLists() throws Exception {
        return (SubjectList[]) loadTestEntities(SUBJECTS_LISTS_FILE_NAME, SubjectList.class);
    }

    public static Teacher[] loadTeachers() throws Exception {
        return (Teacher[]) loadTestEntities(TEACHERS_FILE_NAME, Teacher.class);
    }

    @Test
    protected void loadTestAbsences() throws Exception {
        Absence[] a = loadAbsences();

        Assertions.assertEquals("STUDENT", a[0].getStudent().getFirstName());
        Assertions.assertEquals("A", a[0].getStudent().getLastName());

        Assertions.assertEquals("STUDENT", a[1].getStudent().getFirstName());
        Assertions.assertEquals("B", a[1].getStudent().getLastName());
    }

    @Test
    protected void loadTestBuildings() throws Exception {
        Building[] b = loadBuildings();

        Assertions.assertEquals("BUILDING A", b[0].getBuildingName());
        Assertions.assertEquals("BUILDING B", b[1].getBuildingName());
    }

    @Test
    protected void loadTestClasses() throws Exception {
        SchoolClass[] s = loadClasses();

        Assertions.assertEquals("CLASS A", s[0].getClassName());
        Assertions.assertEquals("ROOM A", s[0].getRoom().getRoomName());
        Assertions.assertEquals("TEACHER", s[0].getTeacher().getFirstName());
        Assertions.assertEquals("A", s[0].getTeacher().getLastName());

        Assertions.assertEquals("CLASS B", s[1].getClassName());
        Assertions.assertEquals("ROOM B", s[1].getRoom().getRoomName());
        Assertions.assertEquals("TEACHER", s[1].getTeacher().getFirstName());
        Assertions.assertEquals("B", s[1].getTeacher().getLastName());
    }

    @Test
    protected void loadTestEvaluations() throws Exception {
        Evaluation[] e = loadEvaluations();

        Assertions.assertEquals("TEST A", e[0].getTestName());
        Assertions.assertEquals(1, e[0].getGrade());
        Assertions.assertEquals(2, e[0].getWeight());

        Assertions.assertEquals("TEST B", e[1].getTestName());
        Assertions.assertEquals(3, e[1].getGrade());
        Assertions.assertEquals(4, e[1].getWeight());
    }

    @Test
    protected void loadTestRooms() throws Exception {
        Room[] r = loadRooms();

        Assertions.assertEquals("ROOM A", r[0].getRoomName());
        Assertions.assertEquals(10, r[0].getCapacity());
        Assertions.assertEquals("BUILDING A", r[0].getBuilding().getBuildingName());

        Assertions.assertEquals("ROOM B", r[1].getRoomName());
        Assertions.assertEquals(20, r[1].getCapacity());
        Assertions.assertEquals("BUILDING A", r[1].getBuilding().getBuildingName());
    }

    @Test
    protected void loadTestScheduleActionsDates() throws Exception {
        ScheduleActionDate[] s = loadScheduleActionDates();

        Assertions.assertEquals("2022-01-01", s[0].getDate());
        Assertions.assertEquals("THEME A", s[0].getTheme());

        Assertions.assertEquals("2022-02-02", s[1].getDate());
        Assertions.assertEquals("THEME B", s[1].getTheme());
    }

    @Test
    protected void loadTestScheduleActions() throws Exception {
        ScheduleAction[] s = loadScheduleActions();

        Assertions.assertEquals("ROOM A", s[0].getRoom().getRoomName());
        Assertions.assertEquals("SUBJECT A", s[0].getSubject().getSubjectName());
        Assertions.assertEquals("DAY A", s[0].getSchoolday().getSchooldayName());
        Assertions.assertEquals("HOUR A", s[0].getSchoolhour().getSchoolhourName());

        Assertions.assertEquals("ROOM B", s[1].getRoom().getRoomName());
        Assertions.assertEquals("SUBJECT B", s[1].getSubject().getSubjectName());
        Assertions.assertEquals("DAY B", s[1].getSchoolday().getSchooldayName());
        Assertions.assertEquals("HOUR B", s[1].getSchoolhour().getSchoolhourName());
    }

    @Test
    protected void loadTestSchoolDays() throws Exception {
        SchoolDay[] s = loadSchoolDays();

        Assertions.assertEquals("DAY A", s[0].getSchooldayName());
        Assertions.assertEquals("DAY B", s[1].getSchooldayName());
    }

    @Test
    protected void loadTestSchoolHours() throws Exception {
        SchoolHour[] s = loadSchoolHours();

        Assertions.assertEquals("HOUR A", s[0].getSchoolhourName());
        Assertions.assertEquals("8:00:00", s[0].getSchoolhourBegin());
        Assertions.assertEquals("9:00:00", s[0].getSchoolhourEnd());

        Assertions.assertEquals("HOUR B", s[1].getSchoolhourName());
        Assertions.assertEquals("9:00:00", s[1].getSchoolhourBegin());
        Assertions.assertEquals("10:00:00", s[1].getSchoolhourEnd());
    }

    @Test
    protected void loadTestStudents() throws Exception {
        Student[] s = loadStudents();

        Assertions.assertEquals("STUDENT", s[0].getFirstName());
        Assertions.assertEquals("A", s[0].getLastName());
        Assertions.assertEquals("2000-01-01", s[0].getBirthDate());
        Assertions.assertEquals("RESIDENCE A", s[0].getResidence());
        Assertions.assertEquals(123456789, s[0].getPhone());
        Assertions.assertEquals("A@test.te", s[0].getEmail());
        Assertions.assertEquals("CLASS A", s[0].getClassesEntity().getClassName());

        Assertions.assertEquals("STUDENT", s[1].getFirstName());
        Assertions.assertEquals("B", s[1].getLastName());
        Assertions.assertEquals("2001-01-01", s[1].getBirthDate());
        Assertions.assertEquals("RESIDENCE B", s[1].getResidence());
        Assertions.assertEquals(987654321, s[1].getPhone());
        Assertions.assertEquals("B@test.te", s[1].getEmail());
        Assertions.assertEquals("CLASS B", s[1].getClassesEntity().getClassName());
    }

    @Test
    protected void loadTestSubjects() throws Exception {
        Subject[] s = loadSubjects();

        Assertions.assertEquals("SUBJECT A", s[0].getSubjectName());
        Assertions.assertEquals("SUBJECT LIST A", s[0].getSubjectlist().getSubjectName());
        Assertions.assertEquals("TEACHER", s[0].getTeacher().getFirstName());
        Assertions.assertEquals("A", s[0].getTeacher().getLastName());

        Assertions.assertEquals("SUBJECT B", s[1].getSubjectName());
        Assertions.assertEquals("SUBJECT LIST A", s[1].getSubjectlist().getSubjectName());
        Assertions.assertEquals("TEACHER", s[1].getTeacher().getFirstName());
        Assertions.assertEquals("B", s[1].getTeacher().getLastName());
    }

    @Test
    protected void loadTestSubjectsLists() throws Exception {
        SubjectList[] s = loadSubjectsLists();

        Assertions.assertEquals("SUBJECT LIST A", s[0].getSubjectName());
        Assertions.assertEquals("SUBJECT LIST B", s[1].getSubjectName());
    }

    @Test
    protected void loadTestTeachers() throws Exception {
        Teacher[] t = loadTeachers();

        Assertions.assertEquals("TEACHER", t[0].getFirstName());
        Assertions.assertEquals("A", t[0].getLastName());
        Assertions.assertEquals("1960-01-01", t[0].getBirthDate());
        Assertions.assertEquals("2022-01-01", t[0].getCommencementDate());
        Assertions.assertEquals("RESIDENCE A", t[0].getResidence());
        Assertions.assertEquals(123546789, t[0].getPhone());
        Assertions.assertEquals("A@test.te", t[0].getEmail());

        Assertions.assertEquals("TEACHER", t[1].getFirstName());
        Assertions.assertEquals("B", t[1].getLastName());
        Assertions.assertEquals("1960-02-01", t[1].getBirthDate());
        Assertions.assertEquals("2022-02-01", t[1].getCommencementDate());
        Assertions.assertEquals("RESIDENCE B", t[1].getResidence());
        Assertions.assertEquals(987654321, t[1].getPhone());
        Assertions.assertEquals("B@test.te", t[1].getEmail());
    }
}
